#ifndef _SCANNER_H
#define _SCANNER_H

/*
 *  Copyright (c) Thomas Faehnle 1994  (Thomas.Faehnle@student.uni-ulm.de)
 *
 *  scanner.h,v 1.4 1994/04/13 21:23:40 blurf Exp
 */


struct m105_modeinfo_struct {
  unsigned short dpi;		/* resolution */
  unsigned short bpl;		/* bytes per scanline */
};
  
struct gi1904_regs_struct {
  unsigned char mr0;
  unsigned char mr1;
  unsigned char mr2;
  unsigned char ctl_mask;
};
					/* magic ioctl numbers */

#define HSIOCGMOD		0x73ea
#define HSIOCSMOD		0x73eb  /* get/(set) modeinfo struct */
#define HSIOCGREG		0x73ec	/* play with strange registers */ 
#define HSIOCSREG		0x73ef
#define HSIOCGBUF		0x73f0	/* get/set current buffer size (scanlines) */
#define HSIOCSBUF		0x73f1
#define HSIOCSSTH		0x73f2  /* get/set select(2) threshold (scanlines) */
#define HSIOCGSTH		0x73f3

#endif /* _SCANNER_H */
